@@ -6,7 +6,7 @@ Apache::Voodoo::Install::Updater
 
 =head1 VERSION
 
-$Id: Distribution.pm 4342 2006-12-18 23:21:06Z medwards $
+$Id: Distribution.pm 6538 2008-02-04 19:11:38Z medwards $
 
 =head1 SYNOPSIS
 
@@ -16,8 +16,6 @@ by the various .xml files in an application.
 =cut ###########################################################################
 package Apache::Voodoo::Install::Distribution;
 
-$VERSION = '1.21';
-
 use strict;
 use warnings;
 
@@ -43,21 +41,21 @@ sub new {
 
 	$self->{'app_name'} = $self->{'distribution'};
 	$self->{'app_name'} =~ s/\.tar\.(bz2|gz)$//i;
-	$self->{'app_name'} =~ s/-.*$//;
+	$self->{'app_name'} =~ s/-[\d.]*(-beta\d+)?$//;
 	$self->{'app_name'} =~ s/.*\///;
 
-	unless ($self->{'app_name'} =~ /^[a-z]\w*$/i) {
+	unless ($self->{'app_name'} =~ /^[a-z][\w-]*$/i) {
 		die "ERROR: Distribution file names must follow the format: AppName-Version.tar.(gz|bz2)\n";
 	}
 
 	my $ac = Apache::Voodoo::Constants->new();
 	$self->{'ac'} = $ac;
 
-	$self->{'install_path'} = $ac->install_path()."/".$self->{'app_name'};
+	$self->{'install_path'} = File::Spec->catfile($ac->install_path(),$self->{'app_name'});
 
-	$self->{'conf_file'}    = $self->{'install_path'}."/".$ac->conf_file();
-	$self->{'conf_path'}    = $self->{'install_path'}."/".$ac->conf_path();
-	$self->{'updates_path'} = $self->{'install_path'}."/".$ac->updates_path();
+	$self->{'conf_file'}    = File::Spec->catfile($self->{'install_path'},$ac->conf_file());
+	$self->{'conf_path'}    = File::Spec->catfile($self->{'install_path'},$ac->conf_path());
+	$self->{'updates_path'} = File::Spec->catfile($self->{'install_path'},$ac->updates_path());
 	$self->{'apache_uid'}   = $ac->apache_uid();
 	$self->{'apache_gid'}   = $ac->apache_gid();
 
@@ -92,9 +90,34 @@ sub do_install {
 	my $self = shift;
 
 	$self->unpack_distribution();
-	$self->check_existing();
-	$self->update_conf_file();
-	$self->install_files();
+	my $new_conf = File::Spec->catfile($self->{'unpack_dir'},$self->{'ac'}->conf_file());
+
+	my $pm_dir = $self->{distribution};
+	$pm_dir =~ s/(\.tar(\.gz|\.bz2)?|\.tgz)$//;
+	$pm_dir =~ s/^.*\///;
+	$pm_dir = File::Spec->catfile($self->{'unpack_dir'},$pm_dir);
+
+	if (-e $new_conf) {
+		$self->check_existing();
+		$self->update_conf_file();
+		$self->install_files();
+	}
+	elsif (-e File::Spec->catfile($pm_dir,'Makefile.PL') ||
+	       -e File::Spec->catfile($pm_dir,'Build.PL')) {
+
+		$self->info("This appears to be a standard Perl module. Calling CPAN to install it...\n");
+
+		eval {
+			use CPAN;
+			CPAN::Shell->install(File::Spec->catfile($pm_dir,"."));
+		};
+
+		exit;
+	}
+	else {
+		print "This distribution doesn't follow a format that I know how to handle.  Giving up.\n";
+		exit;
+	}
 }
 
 ################################################################################
@@ -124,13 +147,6 @@ sub unpack_distribution {
 	}
 
 	$self->{'unpack_dir'} = $unpack_dir;
-
-	my $new_conf = $self->{'unpack_dir'}."/".$self->{'ac'}->conf_file();
-
-	unless (-e $new_conf) {
-		print "ERROR: install doesn't contain a configuration file at: $new_conf\n";
-		exit;
-	}
 }
 
 ################################################################################
@@ -150,7 +166,7 @@ sub check_existing {
 		my %old_cdata = $old_config->getall();
 
 		# save old (maybe customized?) config variables
-		foreach ('session_dir','devel_mode','shared_cache','ipc_max_size', 'debug','devel_mode','cookie_name','database') {
+		foreach ('session_dir','devel_mode','debug','devel_mode','cookie_name','database') {
 			$self->{'old_conf_data'}->{$_} = $old_cdata{$_};
 		}
 
@@ -176,7 +192,7 @@ sub check_existing {
 sub update_conf_file {
 	my $self = shift;
 
-	my $new_conf = $self->{'unpack_dir'}."/".$self->{'ac'}->conf_file();
+	my $new_conf = File::Spec->catfile($self->{'unpack_dir'},$self->{'ac'}->conf_file());
 
 	my $config = Config::General->new($new_conf);
 	my %cdata = $config->getall();
